"""
Type annotations for awscrt module.

Copyright 2024 Vlad Emelianov
"""

from awscrt import aio as aio
from awscrt import auth as auth
from awscrt import crypto as crypto
from awscrt import http as http
from awscrt import io as io
from awscrt import mqtt as mqtt
from awscrt import mqtt5 as mqtt5
from awscrt import mqtt_request_response as mqtt_request_response
from awscrt import s3 as s3
from awscrt import websocket as websocket

__all__ = [
    "aio",
    "auth",
    "crypto",
    "http",
    "io",
    "mqtt",
    "mqtt5",
    "mqtt_request_response",
    "s3",
    "websocket",
]
__version__: str = ...

class NativeResource:
    def __init__(self) -> None: ...



"""
Type annotations for awscrt.eventstream.rpc module.

Copyright 2024 Vlad Emelianov
"""

import abc
from abc import ABC, abstractmethod
from concurrent.futures import Future
from enum import IntEnum
from typing import Any, Callable, Sequence

from awscrt import NativeResource
from awscrt.eventstream import Header
from awscrt.http import HttpClientConnection
from awscrt.io import ClientBootstrap, SocketOptions, TlsConnectionOptions

__all__ = [
    "ClientConnection",
    "ClientConnectionHandler",
    "ClientContinuation",
    "ClientContinuationHandler",
    "MessageFlag",
    "MessageType",
]

class MessageType(IntEnum):
    APPLICATION_MESSAGE = 0
    APPLICATION_ERROR = 1
    PING = 2
    PING_RESPONSE = 3
    CONNECT = 4
    CONNECT_ACK = 5
    PROTOCOL_ERROR = 6
    INTERNAL_ERROR = 7
    def __format__(self, format_spec: str) -> str: ...

class MessageFlag:
    NONE: int = ...
    CONNECTION_ACCEPTED: int = ...
    TERMINATE_STREAM: int = ...
    def __format__(self, format_spec: str) -> str: ...

class ClientConnectionHandler(ABC, metaclass=abc.ABCMeta):
    @abstractmethod
    def on_connection_setup(
        self,
        connection: HttpClientConnection | None,
        error: BaseException | None,
        **kwargs: Any,
    ) -> None: ...
    @abstractmethod
    def on_connection_shutdown(self, reason: BaseException | None, **kwargs: Any) -> None: ...
    @abstractmethod
    def on_protocol_message(
        self,
        headers: Sequence[Header],
        payload: bytes,
        message_type: MessageType,
        flags: int,
        **kwargs: Any,
    ) -> None: ...

class ClientConnection(NativeResource):
    def __init__(self, host_name: str, port: int, handler: ClientConnectionHandler) -> None:
        self.host_name: str
        self.port: int
        self.shutdown_future: Future[None]

    @classmethod
    def connect(
        cls,
        *,
        handler: ClientConnectionHandler,
        host_name: str,
        port: int,
        bootstrap: ClientBootstrap | None = ...,
        socket_options: SocketOptions | None = ...,
        tls_connection_options: TlsConnectionOptions | None = ...,
    ) -> Future[BaseException | None]: ...
    def close(self) -> Future[BaseException | None]: ...
    def is_open(self) -> bool: ...
    def send_protocol_message(
        self,
        *,
        headers: Sequence[Header] | None = ...,
        payload: bytes | None = ...,
        message_type: MessageType,
        flags: int | None = ...,
        on_flush: Callable[..., Any] | None = ...,
    ) -> Future[BaseException | None]: ...
    def new_stream(self, handler: ClientContinuationHandler) -> ClientContinuation: ...

class ClientContinuation(NativeResource):
    def __init__(self, handler: ClientConnectionHandler, connection: ClientConnection) -> None:
        self.connection: ClientConnection
        self.closed_future: Future[None]

    def activate(
        self,
        *,
        operation: str,
        headers: Sequence[Header] | None = ...,
        payload: bytes | None = ...,
        message_type: MessageType,
        flags: int | None = ...,
        on_flush: Callable[..., Any] | None = ...,
    ) -> Future[BaseException | None]: ...
    def send_message(
        self,
        *,
        headers: Sequence[Header] | None = ...,
        payload: bytes | None = ...,
        message_type: MessageType,
        flags: int | None = ...,
        on_flush: Callable[..., Any] | None = ...,
    ) -> Future[BaseException | None]: ...
    def is_closed(self) -> bool: ...

class ClientContinuationHandler(ABC, metaclass=abc.ABCMeta):
    @abstractmethod
    def on_continuation_message(
        self,
        headers: Sequence[Header],
        payload: bytes,
        message_type: MessageType,
        flags: int,
        **kwargs: Any,
    ) -> None: ...
    @abstractmethod
    def on_continuation_closed(self, **kwargs: Any) -> None: ...



"""
Type annotations for awscrt.cbor module.

Copyright 2025 Vlad Emelianov
"""

from enum import IntEnum
from typing import Any, Callable

from awscrt import NativeResource

class AwsCborType(IntEnum):
    Unknown = 0
    UnsignedInt = 1
    NegativeInt = 2
    Float = 3
    Bytes = 4
    Text = 5
    ArrayStart = 6
    MapStart = 7
    Tag = 8
    Bool = 9
    Null = 10
    Undefined = 11
    Break = 12
    IndefBytes = 13
    IndefStr = 14
    IndefArray = 15
    IndefMap = 16

class AwsCborEncoder(NativeResource):
    def __init__(self) -> None: ...
    def get_encoded_data(self) -> bytes: ...
    def reset(self) -> None: ...
    def write_int(self, val: int) -> None: ...
    def write_float(self, val: float) -> None: ...
    def write_bytes(self, val: bytes) -> None: ...
    def write_text(self, val: str) -> None: ...
    def write_array_start(self, number_entries: int) -> None: ...
    def write_map_start(self, number_entries: int) -> None: ...
    def write_tag(self, tag_number: int) -> None: ...
    def write_null(self) -> None: ...
    def write_undefined(self) -> None: ...
    def write_indef_array_start(self) -> None: ...
    def write_indef_map_start(self) -> None: ...
    def write_indef_bytes_start(self) -> None: ...
    def write_indef_text_start(self) -> None: ...
    def write_break(self) -> None: ...
    def write_bool(self, val: bool) -> None: ...
    def write_epoch_time(self, val: float) -> None: ...
    def write_list(self, val: list[Any]) -> None: ...
    def write_dict(self, val: dict[Any, Any]) -> None: ...
    def write_data_item(self, data_item: Any) -> None: ...

class AwsCborDecoder(NativeResource):
    def __init__(
        self, src: bytes, on_epoch_time: Callable[[float], Any] | None = ..., **kwargs: Any
    ) -> None: ...
    def _on_epoch_time_callback(self, epoch_secs: float) -> Any: ...
    def peek_next_type(self) -> AwsCborType: ...
    def get_remaining_bytes_len(self) -> int: ...
    def get_remaining_bytes(self) -> bytes: ...
    def reset_src(self, src: bytes) -> None: ...
    def consume_next_single_element(self) -> None: ...
    def consume_next_whole_data_item(self) -> None: ...
    def pop_next_unsigned_int(self) -> int: ...
    def pop_next_negative_int(self) -> int: ...
    def pop_next_double(self) -> float: ...
    def pop_next_bool(self) -> bool: ...
    def pop_next_bytes(self) -> bytes: ...
    def pop_next_text(self) -> str: ...
    def pop_next_array_start(self) -> int: ...
    def pop_next_map_start(self) -> int: ...
    def pop_next_tag_val(self) -> int: ...
    def pop_next_list(self) -> list[Any]: ...
    def pop_next_map(self) -> dict[Any, Any]: ...
    def pop_next_data_item(self) -> Any: ...



"""
Type annotations for awscrt.checksums module.

Copyright 2024 Vlad Emelianov
"""

def crc32(input: bytes, previous_crc32: int = ...) -> int: ...
def crc32c(input: bytes, previous_crc32c: int = ...) -> int: ...
def crc64nvme(input: bytes, previous_crc64nvme: int = 0) -> int: ...
def combine_crc32(crc32_result1: int, crc32_result2: int, data_length2: int) -> int: ...
def combine_crc32c(crc32c_result1: int, crc32c_result2: int, data_length2: int) -> int: ...
def combine_crc64nvme(crc64nvme_result1: int, crc64nvme_result2: int, data_length2: int) -> int: ...



"""
Type annotations for awscrt._test module.

Copyright 2024 Vlad Emelianov
"""

from awscrt import NativeResource as NativeResource
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import DefaultHostResolver as DefaultHostResolver
from awscrt.io import EventLoopGroup as EventLoopGroup

def native_memory_usage() -> int: ...
def dump_native_memory() -> None: ...
def join_all_native_threads(*, timeout_sec: float = ...) -> bool: ...
def check_for_leaks(*, timeout_sec: float = ...) -> None: ...



"""
Type annotations for awscrt.auth module.

Copyright 2024 Vlad Emelianov
"""

from concurrent.futures import Future
from datetime import datetime
from enum import IntEnum
from typing import Any, Callable, Sequence, TypeVar

from awscrt import NativeResource as NativeResource
from awscrt.http import HttpProxyOptions
from awscrt.http import HttpRequest as HttpRequest
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import ClientTlsContext

_R = TypeVar("_R")

class AwsCredentials(NativeResource):
    def __init__(
        self,
        access_key_id: str,
        secret_access_key: str,
        session_token: str | None = ...,
        expiration: datetime | None = ...,
    ) -> None: ...
    @property
    def access_key_id(self) -> str: ...
    @property
    def secret_access_key(self) -> str: ...
    @property
    def session_token(self) -> str: ...
    @property
    def expiration(self) -> datetime | None: ...
    def __deepcopy__(self: _R, memo: Any) -> _R: ...

class AwsCredentialsProviderBase(NativeResource): ...

class AwsCredentialsProvider(AwsCredentialsProviderBase):
    def __init__(self, binding: Any) -> None: ...
    @classmethod
    def new_default_chain(cls: type[_R], client_bootstrap: ClientBootstrap | None = ...) -> _R: ...
    @classmethod
    def new_static(
        cls: type[_R],
        access_key_id: str,
        secret_access_key: str,
        session_token: str | None = ...,
    ) -> _R: ...
    @classmethod
    def new_profile(
        cls: type[_R],
        client_bootstrap: ClientBootstrap | None = ...,
        profile_name: str | None = ...,
        config_filepath: str | None = ...,
        credentials_filepath: str | None = ...,
    ) -> _R: ...
    @classmethod
    def new_process(cls: type[_R], profile_to_use: str | None = ...) -> _R: ...
    @classmethod
    def new_environment(cls: type[_R]) -> _R: ...
    @classmethod
    def new_chain(cls: type[_R], providers: list[AwsCredentialsProvider]) -> _R: ...
    @classmethod
    def new_delegate(cls: type[_R], get_credentials: Callable[[], AwsCredentials]) -> _R: ...
    @classmethod
    def new_cognito(
        cls: type[_R],
        *,
        endpoint: str,
        identity: str,
        tls_ctx: ClientTlsContext,
        logins: Sequence[tuple[str, str]] | None = ...,
        custom_role_arn: str | None = ...,
        client_bootstrap: ClientBootstrap | None = ...,
        http_proxy_options: HttpProxyOptions | None = ...,
    ) -> _R: ...
    @classmethod
    def new_x509(
        cls: type[_R],
        *,
        endpoint: str,
        thing_name: str,
        role_alias: str,
        tls_ctx: ClientTlsContext,
        client_bootstrap: ClientBootstrap | None = ...,
        http_proxy_options: HttpProxyOptions | None = ...,
    ) -> _R: ...
    def get_credentials(self) -> Future[AwsCredentials]: ...

class AwsSigningAlgorithm(IntEnum):
    V4 = 0
    V4_ASYMMETRIC = 1
    V4_S3EXPRESS = 2

class AwsSignatureType(IntEnum):
    HTTP_REQUEST_HEADERS = 0
    HTTP_REQUEST_QUERY_PARAMS = 1

class AwsSignedBodyValue:
    EMPTY_SHA256: str
    UNSIGNED_PAYLOAD: str
    STREAMING_AWS4_HMAC_SHA256_PAYLOAD: str
    STREAMING_AWS4_HMAC_SHA256_EVENTS: str

class AwsSignedBodyHeaderType(IntEnum):
    NONE = 0
    X_AMZ_CONTENT_SHA_256 = 1

class AwsSigningConfig(NativeResource):
    def __init__(
        self,
        algorithm: AwsSigningAlgorithm = ...,
        signature_type: AwsSignatureType = ...,
        credentials_provider: AwsCredentialsProvider | None = ...,
        region: str = ...,
        service: str = ...,
        date: datetime | None = ...,
        should_sign_header: Callable[[str], bool] | None = ...,
        use_double_uri_encode: bool = ...,
        should_normalize_uri_path: bool = ...,
        signed_body_value: str | None = ...,
        signed_body_header_type: AwsSignedBodyHeaderType = ...,
        expiration_in_seconds: int | None = ...,
        omit_session_token: bool = ...,
    ) -> None: ...
    def replace(self: _R, **kwargs: Any) -> _R: ...
    @property
    def algorithm(self) -> AwsSigningAlgorithm: ...
    @property
    def signature_type(self) -> AwsSignatureType: ...
    @property
    def credentials_provider(self) -> AwsCredentialsProvider: ...
    @property
    def region(self) -> str: ...
    @property
    def service(self) -> str: ...
    @property
    def date(self) -> datetime: ...
    @property
    def should_sign_header(self) -> Callable[[str], bool] | None: ...
    @property
    def use_double_uri_encode(self) -> bool: ...
    @property
    def should_normalize_uri_path(self) -> bool: ...
    @property
    def signed_body_value(self) -> str | None: ...
    @property
    def signed_body_header_type(self) -> AwsSignedBodyHeaderType: ...
    @property
    def expiration_in_seconds(self) -> int | None: ...
    @property
    def omit_session_token(self) -> bool: ...

def aws_sign_request(
    http_request: HttpRequest, signing_config: AwsSigningConfig
) -> Future[HttpRequest]: ...



"""
Type annotations for awscrt.websocket module.

Copyright 2024 Vlad Emelianov
"""

from dataclasses import dataclass
from enum import IntEnum
from typing import Any, Callable, Sequence

from awscrt import NativeResource as NativeResource
from awscrt.http import HttpProxyOptions as HttpProxyOptions
from awscrt.http import HttpRequest as HttpRequest
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import SocketOptions as SocketOptions
from awscrt.io import TlsConnectionOptions as TlsConnectionOptions

class Opcode(IntEnum):
    CONTINUATION = 0x0
    TEXT = 0x1
    BINARY = 0x2
    CLOSE = 0x8
    PING = 0x9
    PONG = 0xA
    def is_data_frame(self) -> bool: ...

MAX_PAYLOAD_LENGTH: int

@dataclass
class OnConnectionSetupData:
    exception: Exception | None = ...
    websocket: WebSocket | None = ...
    handshake_response_status: int | None = ...
    handshake_response_headers: Sequence[tuple[str, str]] | None = ...
    handshake_response_body: bytes | None = ...

@dataclass
class OnConnectionShutdownData:
    exception: Exception | None = ...

@dataclass
class IncomingFrame:
    opcode: Opcode
    payload_length: int
    fin: bool
    def is_data_frame(self) -> bool: ...

@dataclass
class OnIncomingFrameBeginData:
    frame: IncomingFrame

@dataclass
class OnIncomingFramePayloadData:
    frame: IncomingFrame
    data: bytes

@dataclass
class OnIncomingFrameCompleteData:
    frame: IncomingFrame
    exception: Exception | None = ...

@dataclass
class OnSendFrameCompleteData:
    exception: Exception | None = ...

class WebSocket(NativeResource):
    def __init__(self, binding: Any) -> None: ...
    def close(self) -> None: ...
    def send_frame(
        self,
        opcode: Opcode,
        payload: str | bytes | bytearray | memoryview | None = ...,
        *,
        fin: bool = ...,
        on_complete: Callable[[OnSendFrameCompleteData], None] | None = ...,
    ) -> None: ...
    def increment_read_window(self, size: int) -> None: ...

class _WebSocketCore(NativeResource):
    def __init__(
        self,
        on_connection_setup: Callable[[OnConnectionSetupData], None],
        on_connection_shutdown: Callable[[OnConnectionShutdownData], None] | None,
        on_incoming_frame_begin: Callable[[OnIncomingFrameBeginData], None] | None,
        on_incoming_frame_payload: Callable[[OnIncomingFramePayloadData], None] | None,
        on_incoming_frame_complete: Callable[[OnIncomingFrameCompleteData], None] | None,
    ) -> None: ...

def connect(
    *,
    host: str,
    port: int | None = ...,
    handshake_request: HttpRequest,
    bootstrap: ClientBootstrap | None = ...,
    socket_options: SocketOptions | None = ...,
    tls_connection_options: TlsConnectionOptions | None = ...,
    proxy_options: HttpProxyOptions | None = ...,
    manage_read_window: bool = ...,
    initial_read_window: int | None = ...,
    on_connection_setup: Callable[[OnConnectionSetupData], None],
    on_connection_shutdown: Callable[[OnConnectionShutdownData], None] | None = ...,
    on_incoming_frame_begin: Callable[[OnIncomingFrameBeginData], None] | None = ...,
    on_incoming_frame_payload: Callable[[OnIncomingFramePayloadData], None] | None = ...,
    on_incoming_frame_complete: Callable[[OnIncomingFrameCompleteData], None] | None = ...,
) -> None: ...
def create_handshake_request(*, host: str, path: str = ...) -> HttpRequest: ...



"""
Type annotations for awscrt.http module.

Copyright 2024 Vlad Emelianov
"""

from concurrent.futures import Future
from enum import IntEnum
from typing import IO, Any, Callable, Iterator, TypeVar

from awscrt import NativeResource as NativeResource
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import InputStream as InputStream
from awscrt.io import SocketOptions as SocketOptions
from awscrt.io import TlsConnectionOptions as TlsConnectionOptions

_R = TypeVar("_R")

class HttpVersion(IntEnum):
    Unknown = 0
    Http1_0 = 1
    Http1_1 = 2
    Http2 = 3

class Http2SettingID(IntEnum):
    HEADER_TABLE_SIZE = 1
    ENABLE_PUSH = 2
    MAX_CONCURRENT_STREAMS = 3
    INITIAL_WINDOW_SIZE = 4
    MAX_FRAME_SIZE = 5
    MAX_HEADER_LIST_SIZE = 6

class Http2Setting:
    VALID_RANGES: dict[Http2SettingID, tuple[int, int]] = ...
    def __init__(
        self,
        id: Http2SettingID,
        value: int,
    ) -> None:
        self.id: Http2SettingID
        self.value: int

class HttpConnectionBase(NativeResource):
    def __init__(self) -> None: ...
    @property
    def shutdown_future(self) -> Future[None]: ...
    @property
    def version(self) -> str: ...
    def is_open(self) -> bool: ...

class HttpClientConnectionBase(HttpConnectionBase):
    @property
    def host_name(self) -> str: ...
    @property
    def port(self) -> int: ...

class HttpClientConnection(HttpClientConnectionBase):
    @classmethod
    def new(
        cls: type[_R],
        host_name: str,
        port: int,
        bootstrap: ClientBootstrap | None = ...,
        socket_options: SocketOptions | None = ...,
        tls_connection_options: TlsConnectionOptions | None = ...,
        proxy_options: HttpProxyOptions | None = ...,
    ) -> Future[_R]: ...
    def request(
        self,
        request: HttpRequest,
        on_response: Callable[[HttpClientStream, int, list[tuple[str, str]]], None] | None = ...,
        on_body: Callable[[HttpClientStream, bytes], None] | None = ...,
    ) -> HttpClientStream: ...
    def close(self) -> Future[None]: ...

class Http2ClientConnection(HttpClientConnectionBase):
    @classmethod
    def new(  # type: ignore[override]
        cls,
        host_name: str,
        port: int,
        bootstrap: ClientBootstrap | None = ...,
        socket_options: SocketOptions | None = ...,
        tls_connection_options: TlsConnectionOptions | None = ...,
        proxy_options: HttpProxyOptions | None = ...,
        initial_settings: list[Http2Setting] | None = ...,
        on_remote_settings_changed: Callable[[list[Http2Setting]], None] | None = ...,
    ) -> Future[HttpClientConnection]: ...
    def request(
        self,
        request: HttpRequest,
        on_response: Callable[[HttpClientStream, int, list[tuple[str, str]]], None] | None = ...,
        on_body: Callable[[HttpClientStream, bytes], None] | None = ...,
        manual_write: bool = ...,
    ) -> Http2ClientStream: ...
    def close(self) -> Future[None]: ...

class HttpStreamBase(NativeResource):
    def __init__(
        self,
        connection: HttpClientConnection,
        on_body: Callable[[HttpClientStream, bytes], None] | None = ...,
    ) -> None: ...
    @property
    def connection(self) -> HttpClientConnection: ...
    @property
    def completion_future(self) -> Future[int]: ...

class HttpClientStreamBase(HttpStreamBase):
    @property
    def version(self) -> HttpVersion: ...
    @property
    def response_status_code(self) -> int: ...

class HttpClientStream(HttpClientStreamBase):
    def __init__(
        self,
        connection: HttpClientConnection,
        request: HttpRequest,
        on_response: Callable[[HttpClientStream, int, list[tuple[str, str]]], None] | None = ...,
        on_body: Callable[[HttpClientStream, bytes], None] | None = ...,
    ) -> None: ...
    def activate(self) -> None: ...

class Http2ClientStream(HttpClientStreamBase):
    def __init__(
        self,
        connection: HttpClientConnection,
        request: HttpRequest,
        on_response: Callable[[HttpClientStream, int, list[tuple[str, str]]], None] | None = ...,
        on_body: Callable[[HttpClientStream, bytes], None] | None = ...,
        manual_write: bool = ...,
    ) -> None: ...
    def activate(self) -> None: ...
    def write_data(self, data_stream: IO[Any], end_stream: bool = ...) -> None: ...

class HttpMessageBase(NativeResource):
    def __init__(
        self, binding: Any, headers: HttpHeaders, body_stream: IO[Any] | None = ...
    ) -> None: ...
    @property
    def headers(self) -> HttpHeaders: ...
    @property
    def body_stream(self) -> IO[Any] | None: ...
    @body_stream.setter
    def body_stream(self, stream: IO[Any]) -> None: ...

class HttpRequest(HttpMessageBase):
    def __init__(
        self,
        method: str = ...,
        path: str = ...,
        headers: HttpHeaders | None = ...,
        body_stream: IO[Any] | None = ...,
    ) -> None: ...
    @property
    def method(self) -> str: ...
    @method.setter
    def method(self, method: str) -> None: ...
    @property
    def path(self) -> str: ...
    @path.setter
    def path(self, path: str) -> None: ...

class HttpHeaders(NativeResource):
    def __init__(self, name_value_pairs: list[tuple[str, str]] | None = ...) -> None: ...
    def add(self, name: str, value: str) -> None: ...
    def add_pairs(self, name_value_pairs: list[tuple[str, str]]) -> None: ...
    def set(self, name: str, value: str) -> None: ...
    def get_values(self, name: str) -> Iterator[tuple[str, str]]: ...
    def get(self, name: str, default: str | None = ...) -> str | None: ...
    def remove(self, name: str) -> None: ...
    def remove_value(self, name: str, value: str) -> None: ...
    def clear(self) -> None: ...
    def __iter__(self) -> Iterator[tuple[str, str]]: ...

class HttpProxyConnectionType(IntEnum):
    Legacy = 0
    Forwarding = 1
    Tunneling = 2

class HttpProxyAuthenticationType(IntEnum):
    Nothing = 0
    Basic = 1

class HttpProxyOptions:
    def __init__(
        self,
        host_name: str,
        port: int,
        tls_connection_options: TlsConnectionOptions | None = ...,
        auth_type: HttpProxyAuthenticationType = ...,
        auth_username: str | None = ...,
        auth_password: str | None = ...,
        connection_type: HttpProxyConnectionType | None = ...,
    ) -> None:
        self.host_name: str
        self.port: int
        self.tls_connection_options: TlsConnectionOptions | None
        self.auth_type: HttpProxyAuthenticationType
        self.auth_username: str | None
        self.auth_password: str | None
        self.connection_type: HttpProxyConnectionType



"""
Type annotations for awscrt.aio.http module.

Copyright 2025 Vlad Emelianov
"""

import asyncio
from typing import AsyncIterator, Callable

from awscrt.http import (
    Http2Setting,
    HttpClientConnectionBase,
    HttpClientStreamBase,
    HttpProxyOptions,
    HttpRequest,
)
from awscrt.io import ClientBootstrap, SocketOptions, TlsConnectionOptions

class AIOHttpClientConnectionUnified(HttpClientConnectionBase):
    @classmethod
    async def new(
        cls,
        host_name: str,
        port: int,
        bootstrap: ClientBootstrap | None = ...,
        socket_options: SocketOptions | None = ...,
        tls_connection_options: TlsConnectionOptions | None = ...,
        proxy_options: HttpProxyOptions | None = ...,
    ) -> AIOHttpClientConnectionUnified: ...
    async def close(self) -> None: ...
    def request(
        self,
        request: HttpRequest,
        request_body_generator: AsyncIterator[bytes] | None = ...,
        loop: asyncio.AbstractEventLoop | None = ...,
    ) -> AIOHttpClientStreamUnified: ...

class AIOHttpClientConnection(AIOHttpClientConnectionUnified):
    @classmethod
    async def new(
        cls,
        host_name: str,
        port: int,
        bootstrap: ClientBootstrap | None = ...,
        socket_options: SocketOptions | None = ...,
        tls_connection_options: TlsConnectionOptions | None = ...,
        proxy_options: HttpProxyOptions | None = ...,
    ) -> AIOHttpClientConnection: ...
    def request(
        self,
        request: HttpRequest,
        request_body_generator: AsyncIterator[bytes] | None = ...,
        loop: asyncio.AbstractEventLoop | None = ...,
    ) -> AIOHttpClientStream: ...

class AIOHttp2ClientConnection(AIOHttpClientConnectionUnified):
    @classmethod
    async def new(
        cls,
        host_name: str,
        port: int,
        bootstrap: ClientBootstrap | None = ...,
        socket_options: SocketOptions | None = ...,
        tls_connection_options: TlsConnectionOptions | None = ...,
        proxy_options: HttpProxyOptions | None = ...,
        initial_settings: list[Http2Setting] | None = ...,
        on_remote_settings_changed: Callable[[list[Http2Setting]], None] | None = ...,
    ) -> AIOHttp2ClientConnection: ...
    def request(
        self,
        request: HttpRequest,
        request_body_generator: AsyncIterator[bytes] | None = ...,
        loop: asyncio.AbstractEventLoop | None = ...,
    ) -> AIOHttp2ClientStream: ...

class AIOHttpClientStreamUnified(HttpClientStreamBase):
    def __init__(
        self,
        connection: AIOHttpClientConnection,
        request: HttpRequest,
        request_body_generator: AsyncIterator[bytes] | None = ...,
        loop: asyncio.AbstractEventLoop | None = ...,
    ) -> None: ...
    async def get_response_status_code(self) -> int: ...
    async def get_response_headers(self) -> list[tuple[str, str]]: ...
    async def get_next_response_chunk(self) -> bytes: ...
    async def wait_for_completion(self) -> int: ...

class AIOHttpClientStream(AIOHttpClientStreamUnified):
    def __init__(
        self,
        connection: AIOHttpClientConnection,
        request: HttpRequest,
        loop: asyncio.AbstractEventLoop | None = ...,
    ) -> None: ...

class AIOHttp2ClientStream(AIOHttpClientStreamUnified):
    def __init__(
        self,
        connection: AIOHttpClientConnection,
        request: HttpRequest,
        request_body_generator: AsyncIterator[bytes] | None = ...,
        loop: asyncio.AbstractEventLoop | None = ...,
    ) -> None: ...



"""
Type annotations for awscrt.common module.

Copyright 2024 Vlad Emelianov
"""

def get_cpu_group_count() -> int: ...
def get_cpu_count_for_group(group_idx: int) -> int: ...
def join_all_native_threads(*, timeout_sec: float = ...) -> bool: ...



"""
Type annotations for awscrt.crypto module.

Copyright 2024 Vlad Emelianov
"""

from enum import IntEnum
from typing import Any, NamedTuple

from awscrt import NativeResource

class Hash:
    def __init__(self, native_handle: Any) -> None: ...
    @staticmethod
    def sha1_new() -> Hash: ...
    @staticmethod
    def sha256_new() -> Hash: ...
    @staticmethod
    def md5_new() -> Hash: ...
    def update(self, to_hash: Any) -> None: ...
    def digest(self, truncate_to: int = ...) -> str: ...

class HMAC:
    def __init__(self, native_handle: Any) -> None: ...
    @staticmethod
    def sha256_hmac_new(secret_key: str) -> HMAC: ...
    def update(self, to_hmac: Any) -> None: ...
    def digest(self, truncate_to: int = ...) -> str: ...

class RSAEncryptionAlgorithm(IntEnum):
    PKCS1_5 = 0
    OAEP_SHA256 = 1
    OAEP_SHA512 = 2

class RSASignatureAlgorithm(IntEnum):
    PKCS1_5_SHA256 = 0
    PKCS1_5_SHA1 = 1
    PSS_SHA256 = 2

class RSA(NativeResource):
    def __init__(self, binding: Any) -> None: ...
    @staticmethod
    def new_private_key_from_pem_data(
        pem_data: str | bytes | bytearray | memoryview,
    ) -> RSA: ...
    @staticmethod
    def new_public_key_from_pem_data(
        pem_data: str | bytes | bytearray | memoryview,
    ) -> RSA: ...
    @staticmethod
    def new_private_key_from_der_data(
        der_data: str | bytes | bytearray | memoryview,
    ) -> RSA: ...
    @staticmethod
    def new_public_key_from_der_data(der_data: bytes | bytearray | memoryview) -> RSA: ...
    def encrypt(
        self,
        encryption_algorithm: RSAEncryptionAlgorithm,
        plaintext: bytes | bytearray | memoryview,
    ) -> bytes: ...
    def decrypt(
        self,
        encryption_algorithm: RSAEncryptionAlgorithm,
        ciphertext: bytes | bytearray | memoryview,
    ) -> bytes: ...
    def sign(
        self,
        signature_algorithm: RSASignatureAlgorithm,
        digest: bytes | bytearray | memoryview,
    ) -> bytes: ...
    def verify(
        self,
        signature_algorithm: RSASignatureAlgorithm,
        digest: bytes | bytearray | memoryview,
        signature: bytes | bytearray | memoryview,
    ) -> bool: ...

class ED25519ExportFormat(IntEnum):
    RAW = 0
    OPENSSH_B64 = 1

class ED25519(NativeResource):
    def __init__(self, binding: Any) -> None: ...
    @staticmethod
    def new_generate() -> ED25519: ...
    def export_public_key(self, export_format: ED25519ExportFormat) -> bytes: ...
    def export_private_key(self, export_format: ED25519ExportFormat) -> bytes: ...

class ECType(IntEnum):
    P_256 = 0
    P_384 = 1

class ECExportFormat(IntEnum):
    SEC1 = 0
    PKCS8 = 1
    SPKI = 2

class ECRawSignature(NamedTuple):
    r: bytes
    s: bytes

class ECPublicCoords(NamedTuple):
    x: bytes
    y: bytes

class EC(NativeResource):
    def __init__(self, binding: Any) -> None: ...
    @staticmethod
    def new_generate(type: ECType) -> EC: ...
    @staticmethod
    def new_key_from_der_data(der_data: bytes | bytearray | memoryview) -> EC: ...
    @staticmethod
    def decode_der_signature(signature: bytes) -> ECRawSignature: ...
    @staticmethod
    def decode_der_signature_to_padded_pair(signature: bytes, pad_to: int) -> bytes: ...
    @staticmethod
    def encode_raw_signature(signature: ECRawSignature) -> bytes: ...
    def export_key(self, export_format: ECExportFormat) -> bytes: ...
    def get_public_coords(self) -> ECPublicCoords: ...
    def sign(self, digest: bytes | bytearray | memoryview) -> bytes: ...
    def verify(
        self, digest: bytes | bytearray | memoryview, signature: bytes | bytearray | memoryview
    ) -> bool: ...



"""
Type annotations for awscrt.mqtt5 module.

Copyright 2024 Vlad Emelianov
"""

from concurrent.futures import Future
from dataclasses import dataclass
from enum import IntEnum
from typing import Any, Callable, Sequence

from awscrt import NativeResource as NativeResource
from awscrt import exceptions as exceptions
from awscrt.exceptions import AwsCrtError
from awscrt.http import HttpProxyOptions as HttpProxyOptions
from awscrt.http import HttpRequest as HttpRequest
from awscrt.io import ClientBootstrap as ClientBootstrap
from awscrt.io import ClientTlsContext as ClientTlsContext
from awscrt.io import SocketOptions as SocketOptions
from awscrt.mqtt import (
    Connection,
    ConnectReturnCode,
    OnConnectionClosedData,
    OnConnectionFailureData,
    OnConnectionSuccessData,
)
from awscrt.mqtt import QoS as Mqtt3QoS

class QoS(IntEnum):
    AT_MOST_ONCE = 0
    AT_LEAST_ONCE = 1
    EXACTLY_ONCE = 2

    def to_mqtt3(self) -> Mqtt3QoS: ...

class ConnectReasonCode(IntEnum):
    SUCCESS = 0
    UNSPECIFIED_ERROR = 128
    MALFORMED_PACKET = 129
    PROTOCOL_ERROR = 130
    IMPLEMENTATION_SPECIFIC_ERROR = 131
    UNSUPPORTED_PROTOCOL_VERSION = 132
    CLIENT_IDENTIFIER_NOT_VALID = 133
    BAD_USERNAME_OR_PASSWORD = 134
    NOT_AUTHORIZED = 135
    SERVER_UNAVAILABLE = 136
    SERVER_BUSY = 137
    BANNED = 138
    BAD_AUTHENTICATION_METHOD = 140
    TOPIC_NAME_INVALID = 144
    PACKET_TOO_LARGE = 149
    QUOTA_EXCEEDED = 151
    PAYLOAD_FORMAT_INVALID = 153
    RETAIN_NOT_SUPPORTED = 154
    QOS_NOT_SUPPORTED = 155
    USE_ANOTHER_SERVER = 156
    SERVER_MOVED = 157
    CONNECTION_RATE_EXCEEDED = 159

class DisconnectReasonCode(IntEnum):
    NORMAL_DISCONNECTION = 0
    DISCONNECT_WITH_WILL_MESSAGE = 4
    UNSPECIFIED_ERROR = 128
    MALFORMED_PACKET = 129
    PROTOCOL_ERROR = 130
    IMPLEMENTATION_SPECIFIC_ERROR = 131
    NOT_AUTHORIZED = 135
    SERVER_BUSY = 137
    SERVER_SHUTTING_DOWN = 139
    KEEP_ALIVE_TIMEOUT = 141
    SESSION_TAKEN_OVER = 142
    TOPIC_FILTER_INVALID = 143
    TOPIC_NAME_INVALID = 144
    RECEIVE_MAXIMUM_EXCEEDED = 147
    TOPIC_ALIAS_INVALID = 148
    PACKET_TOO_LARGE = 149
    MESSAGE_RATE_TOO_HIGH = 150
    QUOTA_EXCEEDED = 151
    ADMINISTRATIVE_ACTION = 152
    PAYLOAD_FORMAT_INVALID = 153
    RETAIN_NOT_SUPPORTED = 154
    QOS_NOT_SUPPORTED = 155
    USE_ANOTHER_SERVER = 156
    SERVER_MOVED = 157
    SHARED_SUBSCRIPTIONS_NOT_SUPPORTED = 158
    CONNECTION_RATE_EXCEEDED = 159
    MAXIMUM_CONNECT_TIME = 160
    SUBSCRIPTION_IDENTIFIERS_NOT_SUPPORTED = 161
    WILDCARD_SUBSCRIPTIONS_NOT_SUPPORTED = 162

class PubackReasonCode(IntEnum):
    SUCCESS = 0
    NO_MATCHING_SUBSCRIBERS = 16
    UNSPECIFIED_ERROR = 128
    IMPLEMENTATION_SPECIFIC_ERROR = 131
    NOT_AUTHORIZED = 135
    TOPIC_NAME_INVALID = 144
    PACKET_IDENTIFIER_IN_USE = 145
    QUOTA_EXCEEDED = 151
    PAYLOAD_FORMAT_INVALID = 153

class SubackReasonCode(IntEnum):
    GRANTED_QOS_0 = 0
    GRANTED_QOS_1 = 1
    GRANTED_QOS_2 = 2
    UNSPECIFIED_ERROR = 128
    IMPLEMENTATION_SPECIFIC_ERROR = 131
    NOT_AUTHORIZED = 135
    TOPIC_FILTER_INVALID = 143
    PACKET_IDENTIFIER_IN_USE = 145
    QUOTA_EXCEEDED = 151
    SHARED_SUBSCRIPTIONS_NOT_SUPPORTED = 158
    SUBSCRIPTION_IDENTIFIERS_NOT_SUPPORTED = 161
    WILDCARD_SUBSCRIPTIONS_NOT_SUPPORTED = 162

class UnsubackReasonCode(IntEnum):
    SUCCESS = 0
    NO_SUBSCRIPTION_EXISTED = 17
    UNSPECIFIED_ERROR = 128
    IMPLEMENTATION_SPECIFIC_ERROR = 131
    NOT_AUTHORIZED = 135
    TOPIC_FILTER_INVALID = 143
    PACKET_IDENTIFIER_IN_USE = 145

class ClientSessionBehaviorType(IntEnum):
    DEFAULT = 0
    CLEAN = 1
    REJOIN_POST_SUCCESS = 2
    REJOIN_ALWAYS = 3

class PayloadFormatIndicator(IntEnum):
    AWS_MQTT5_PFI_BYTES = 0
    AWS_MQTT5_PFI_UTF8 = 1

class RetainHandlingType(IntEnum):
    SEND_ON_SUBSCRIBE = 0
    SEND_ON_SUBSCRIBE_IF_NEW = 1
    DONT_SEND = 2

class RetainAndHandlingType(IntEnum):
    SEND_ON_SUBSCRIBE = 0
    SEND_ON_SUBSCRIBE_IF_NEW = 1
    DONT_SEND = 2

class ExtendedValidationAndFlowControlOptions(IntEnum):
    NONE = 0
    AWS_IOT_CORE_DEFAULTS = 1

class ClientOperationQueueBehaviorType(IntEnum):
    DEFAULT = 0
    FAIL_NON_QOS1_PUBLISH_ON_DISCONNECT = 1
    FAIL_QOS0_PUBLISH_ON_DISCONNECT = 2
    FAIL_ALL_ON_DISCONNECT = 3

class ExponentialBackoffJitterMode(IntEnum):
    DEFAULT = 0
    NONE = 1
    FULL = 2
    DECORRELATED = 3

@dataclass
class UserProperty:
    name: str | None = ...
    value: str | None = ...

class OutboundTopicAliasBehaviorType(IntEnum):
    DEFAULT = 0
    MANUAL = 1
    LRU = 2
    DISABLED = 3

class InboundTopicAliasBehaviorType(IntEnum):
    DEFAULT = 0
    ENABLED = 1
    DISABLED = 2

@dataclass
class TopicAliasingOptions:
    outbound_behavior: OutboundTopicAliasBehaviorType | None = ...
    outbound_cache_max_size: int | None = ...
    inbound_behavior: InboundTopicAliasBehaviorType | None = ...
    inbound_cache_max_size: int | None = ...

@dataclass
class NegotiatedSettings:
    maximum_qos: QoS | None = ...
    session_expiry_interval_sec: int | None = ...
    receive_maximum_from_server: int | None = ...
    maximum_packet_size_to_server: int | None = ...
    topic_alias_maximum_to_server: int | None = ...
    topic_alias_maximum_to_client: int | None = ...
    server_keep_alive_sec: int | None = ...
    retain_available: bool | None = ...
    wildcard_subscriptions_available: bool | None = ...
    subscription_identifiers_available: bool | None = ...
    shared_subscriptions_available: bool | None = ...
    rejoined_session: bool | None = ...
    client_id: str | None = ...

@dataclass
class ConnackPacket:
    session_present: bool | None = ...
    reason_code: ConnectReasonCode | None = ...
    session_expiry_interval_sec: int | None = ...
    receive_maximum: int | None = ...
    maximum_qos: QoS | None = ...
    retain_available: bool | None = ...
    maximum_packet_size: int | None = ...
    assigned_client_identifier: str | None = ...
    topic_alias_maximum: int | None = ...
    reason_string: str | None = ...
    user_properties: Sequence[UserProperty] | None = ...
    wildcard_subscriptions_available: bool | None = ...
    subscription_identifiers_available: bool | None = ...
    shared_subscription_available: bool | None = ...
    server_keep_alive_sec: int | None = ...
    response_information: str | None = ...
    server_reference: str | None = ...

    @property
    def server_keep_alive(self) -> int | None: ...
    @server_keep_alive.setter
    def server_keep_alive(self, value: int | None) -> None: ...

@dataclass
class DisconnectPacket:
    reason_code: DisconnectReasonCode | None = ...
    session_expiry_interval_sec: int | None = ...
    reason_string: str | None = ...
    user_properties: Sequence[UserProperty] | None = ...
    server_reference: str | None = ...

@dataclass
class Subscription:
    topic_filter: str
    qos: QoS | None = ...
    no_local: bool | None = ...
    retain_as_published: bool | None = ...
    retain_handling_type: RetainAndHandlingType | RetainHandlingType | None = ...

@dataclass
class SubscribePacket:
    subscriptions: Sequence[Subscription]
    subscription_identifier: int | None = ...
    user_properties: Sequence[UserProperty] | None = ...

@dataclass
class SubackPacket:
    reason_string: str | None = ...
    user_properties: Sequence[UserProperty] | None = ...
    reason_codes: Sequence[SubackReasonCode] | None = ...

@dataclass
class UnsubscribePacket:
    topic_filters: Sequence[str]
    user_properties: Sequence[UserProperty] | None = ...

@dataclass
class UnsubackPacket:
    reason_string: str | None = ...
    user_properties: Sequence[UserProperty] | None = ...
    reason_codes: Sequence[DisconnectReasonCode] | None = ...

@dataclass
class PublishPacket:
    payload: Any | None = ...
    qos: QoS | None = ...
    retain: bool | None = ...
    topic: str | None = ...
    payload_format_indicator: PayloadFormatIndicator | None = ...
    message_expiry_interval_sec: int | None = ...
    topic_alias: int | None = ...
    response_topic: str | None = ...
    correlation_data_bytes: bytes | str | None = ...
    correlation_data: Any | None = ...
    subscription_identifiers: Sequence[int] | None = ...
    content_type: str | None = ...
    user_properties: Sequence[UserProperty] | None = ...

@dataclass
class PubackPacket:
    reason_code: PubackReasonCode | None = ...
    reason_string: str | None = ...
    user_properties: Sequence[UserProperty] | None = ...

@dataclass
class ConnectPacket:
    keep_alive_interval_sec: int | None = ...
    client_id: str | None = ...
    username: str | None = ...
    password: str | None = ...
    session_expiry_interval_sec: int | None = ...
    request_response_information: bool | None = ...
    request_problem_information: bool | None = ...
    receive_maximum: int | None = ...
    maximum_packet_size: int | None = ...
    will_delay_interval_sec: int | None = ...
    will: PublishPacket | None = ...
    user_properties: Sequence[UserProperty] | None = ...

class WebsocketHandshakeTransformArgs:
    def __init__(self, client: Client, http_request: HttpRequest, done_future: Future[Any]) -> None:
        self.client: Client
        self.http_request: HttpRequest

    def set_done(self, exception: Exception | None = ...) -> None: ...

@dataclass
class PublishReceivedData:
    publish_packet: PublishPacket | None = ...

@dataclass
class OperationStatisticsData:
    incomplete_operation_count: int = ...
    incomplete_operation_size: int = ...
    unacked_operation_count: int = ...
    unacked_operation_size: int = ...

@dataclass
class LifecycleStoppedData: ...

@dataclass
class LifecycleAttemptingConnectData: ...

@dataclass
class LifecycleConnectSuccessData:
    connack_packet: ConnackPacket | None = ...
    negotiated_settings: NegotiatedSettings | None = ...

@dataclass
class LifecycleConnectFailureData:
    connack_packet: ConnackPacket | None = ...
    exception: Exception | None = ...

@dataclass
class LifecycleDisconnectData:
    disconnect_packet: DisconnectPacket | None = ...
    exception: Exception | None = ...

@dataclass
class PublishCompletionData:
    puback: PubackPacket | None = ...

@dataclass
class ClientOptions:
    host_name: str
    port: int | None = ...
    bootstrap: ClientBootstrap | None = ...
    socket_options: SocketOptions | None = ...
    tls_ctx: ClientTlsContext | None = ...
    http_proxy_options: HttpProxyOptions | None = ...
    websocket_handshake_transform: Callable[[WebsocketHandshakeTransformArgs], None] | None = ...
    connect_options: ConnectPacket | None = ...
    session_behavior: ClientSessionBehaviorType | None = ...
    extended_validation_and_flow_control_options: (
        ExtendedValidationAndFlowControlOptions | None
    ) = ...
    offline_queue_behavior: ClientOperationQueueBehaviorType | None = ...
    retry_jitter_mode: ExponentialBackoffJitterMode | None = ...
    min_reconnect_delay_ms: int | None = ...
    max_reconnect_delay_ms: int | None = ...
    min_connected_time_to_reset_reconnect_delay_ms: int | None = ...
    ping_timeout_ms: int | None = ...
    connack_timeout_ms: int | None = ...
    ack_timeout_sec: int | None = ...
    topic_aliasing_options: TopicAliasingOptions | None = ...
    on_publish_callback_fn: Callable[[PublishReceivedData], None] | None = ...
    on_lifecycle_event_stopped_fn: Callable[[LifecycleStoppedData], None] | None = ...
    on_lifecycle_event_attempting_connect_fn: (
        Callable[[LifecycleAttemptingConnectData], None] | None
    ) = ...
    on_lifecycle_event_connection_success_fn: (
        Callable[[LifecycleConnectSuccessData], None] | None
    ) = ...
    on_lifecycle_event_connection_failure_fn: (
        Callable[[LifecycleConnectFailureData], None] | None
    ) = ...
    on_lifecycle_event_disconnection_fn: Callable[[LifecycleDisconnectData], None] | None = ...

class _ClientCore:
    def __init__(self, client_options: ClientOptions) -> None: ...

class Client(NativeResource):
    def __init__(self, client_options: ClientOptions) -> None: ...
    def start(self) -> None: ...
    def stop(self, disconnect_packet: DisconnectPacket | None = ...) -> None: ...
    def publish(self, publish_packet: PublishPacket) -> Future[PublishCompletionData]: ...
    def subscribe(self, subscribe_packet: SubscribePacket) -> Future[SubackPacket]: ...
    def unsubscribe(self, unsubscribe_packet: UnsubscribePacket) -> Future[UnsubackPacket]: ...
    def get_stats(self) -> Future[OperationStatisticsData]: ...
    def new_connection(
        self,
        on_connection_interrupted: Callable[[Connection, AwsCrtError], None] | None = ...,
        on_connection_resumed: Callable[[Connection, ConnectReturnCode, bool], None] | None = ...,
        on_connection_success: Callable[[Connection, OnConnectionSuccessData], None] | None = ...,
        on_connection_failure: Callable[[Connection, OnConnectionFailureData], None] | None = ...,
        on_connection_closed: Callable[[Connection, OnConnectionClosedData], None] | None = ...,
    ) -> Connection: ...



"""
Type annotations for awscrt.eventstream module.

Copyright 2024 Vlad Emelianov
"""

from enum import IntEnum
from typing import Any
from uuid import UUID

__all__ = ["Header", "HeaderType"]

class HeaderType(IntEnum):
    BOOL_TRUE = 0
    BOOL_FALSE = 1
    BYTE = 2
    INT16 = 3
    INT32 = 4
    INT64 = 5
    BYTE_BUF = 6
    STRING = 7
    TIMESTAMP = 8
    UUID = 9
    def __format__(self, format_spec: str) -> str: ...

class Header:
    def __init__(self, name: str, value: Any, header_type: HeaderType) -> None: ...
    @classmethod
    def from_bool(cls, name: str, value: bool) -> Header: ...
    @classmethod
    def from_byte(cls, name: str, value: int) -> Header: ...
    @classmethod
    def from_int16(cls, name: str, value: int) -> Header: ...
    @classmethod
    def from_int32(cls, name: str, value: int) -> Header: ...
    @classmethod
    def from_int64(cls, name: str, value: int) -> Header: ...
    @classmethod
    def from_byte_buf(cls, name: str, value: bytes | bytearray) -> Header: ...
    @classmethod
    def from_string(cls, name: str, value: str) -> Header: ...
    @classmethod
    def from_timestamp(cls, name: str, value: int) -> Header: ...
    @classmethod
    def from_uuid(cls, name: str, value: UUID) -> Header: ...
    @property
    def name(self) -> str: ...
    @property
    def type(self) -> HeaderType: ...
    @property
    def value(self) -> Any: ...
    def value_as_bool(self) -> bool: ...
    def value_as_byte(self) -> int: ...
    def value_as_int16(self) -> int: ...
    def value_as_int32(self) -> int: ...
    def value_as_int64(self) -> int: ...
    def value_as_byte_buf(self) -> bytes | bytearray: ...
    def value_as_string(self) -> str: ...
    def value_as_timestamp(self) -> int: ...
    def value_as_uuid(self) -> UUID: ...



"""
Type annotations for awscrt.mqtt_request_response module.

Copyright 2025 Vlad Emelianov
"""

from collections.abc import Sequence
from concurrent.futures import Future
from dataclasses import dataclass
from enum import IntEnum
from typing import Any, Callable

import typing_extensions
from awscrt import NativeResource, mqtt, mqtt5

class SubscriptionStatusEventType(IntEnum):
    SUBSCRIPTION_ESTABLISHED = 0
    SUBSCRIPTION_LOST = 1
    SUBSCRIPTION_HALTED = 2

@dataclass
class SubscriptionStatusEvent:
    type: SubscriptionStatusEventType | None = ...
    error: Exception | None = ...

@dataclass
class IncomingPublishEvent:
    topic: str
    payload: bytes | None = ...

SubscriptionStatusListener: typing_extensions.TypeAlias = Callable[[SubscriptionStatusEvent], None]
IncomingPublishListener: typing_extensions.TypeAlias = Callable[[IncomingPublishEvent], None]

@dataclass
class StreamingOperationOptions:
    subscription_topic_filter: str
    subscription_status_listener: SubscriptionStatusListener | None = ...
    incoming_publish_listener: IncomingPublishListener | None = ...

    def validate(self) -> None: ...

@dataclass
class Response:
    topic: str
    payload: bytes | None = None

@dataclass
class ResponsePath:
    topic: str
    correlation_token_json_path: str | None = None

    def validate(self) -> None: ...

@dataclass
class RequestOptions:
    subscription_topic_filters: Sequence[str]
    response_paths: Sequence[ResponsePath]
    publish_topic: str
    payload: bytes
    correlation_token: str | None = None

    def validate(self) -> None: ...

@dataclass
class ClientOptions:
    max_request_response_subscriptions: int
    max_streaming_subscriptions: int
    operation_timeout_in_seconds: int | None = 60

    def validate(self) -> None: ...

class Client(NativeResource):
    def __init__(
        self, protocol_client: mqtt5.Client | mqtt.Connection, client_options: ClientOptions
    ) -> None: ...
    def make_request(self, options: RequestOptions) -> Future[Response]: ...
    def create_stream(self, options: StreamingOperationOptions) -> StreamingOperation: ...

class StreamingOperation(NativeResource):
    def __init__(self, binding: Any) -> None: ...
    def open(self) -> None: ...
